@@ -29,15 +29,15 @@
 #include <telepathy-glib/contact.h>
 #include <telepathy-glib/util.h>
 #include <telepathy-glib/dbus.h>
+#include <telepathy-glib/interfaces.h>
 
 #include "vinagre-tube-handler.h"
 #include "vinagre-commands.h"
 #include "vinagre-tab.h"
 #include "vinagre-notebook.h"
 #include "vinagre-debug.h"
-
-#define MC_DBUS_SERVICE "org.freedesktop.Telepathy.MissionControl"
-#define MC_DBUS_SERVICE_PATH "/org/freedesktop/Telepathy/MissionControl"
+#include "vinagre-plugin.h"
+#include "vinagre-plugins-engine.h"
 
 G_DEFINE_TYPE (VinagreTubeHandler, vinagre_tube_handler, G_TYPE_OBJECT);
 
@@ -54,6 +54,8 @@ struct _VinagreTubeHandlerPrivate
   VinagreNotebook *nb;
   gulong signal_disconnect_id;
   gulong signal_invalidated_id;
+  gchar *hostname;
+  guint port;
 };
 
 enum
@@ -76,7 +78,8 @@ vinagre_tube_handler_dispose (GObject *object)
 {
   VinagreTubeHandlerPrivate *priv = VINAGRE_TUBE_HANDLER_GET_PRIVATE(object);
 
-  vinagre_debug_message (DEBUG_TUBE, "-- Destruction of a Tube handler --\n");
+  vinagre_debug_message (DEBUG_TUBE,
+      "-- Destruction of a Tube handler --\n");
 
   if (priv->channel != NULL)
     {
@@ -181,7 +184,8 @@ vinagre_tube_handler_tab_disconnected_cb (GtkNotebook *notebook,
 
   if (priv->tab == tab)
     {
-      vinagre_debug_message (DEBUG_TUBE,"Tab has been destroyed. Closing the tube handler.");
+      vinagre_debug_message (DEBUG_TUBE,
+          "Tab has been destroyed. Closing the tube handler.\n");
       vinagre_tube_handler_fire_disconnected (self);
     }
 }
@@ -195,7 +199,7 @@ vinagre_tube_handler_tube_invalidated (TpProxy *channel,
 {
   VinagreTubeHandlerPrivate *priv = VINAGRE_TUBE_HANDLER_GET_PRIVATE (self);
 
-  vinagre_debug_message (DEBUG_TUBE,"Tube is invalidated : %s\n", message);
+  vinagre_debug_message (DEBUG_TUBE, "Tube is invalidated : %s\n", message);
 
   g_object_unref (priv->channel);
   priv->channel = NULL;
@@ -204,53 +208,90 @@ vinagre_tube_handler_tube_invalidated (TpProxy *channel,
 }
 
 static void
-vinagre_tube_handler_accept_stream_tube_cb (TpChannel *channel,
-    const GValue *address,
+vinagre_tube_handler_call_service_cb (TpProxy *channel,
+    const GValue *service,
     const GError *error,
     gpointer self,
     GObject *weak_object)
 {
-  gchar *hostname;
-  guint port;
+  VinagreTubeHandlerPrivate *priv = VINAGRE_TUBE_HANDLER_GET_PRIVATE (self);
+  gchar *error_msg = NULL;
+  gchar *error_conn_msg = NULL;
+  VinagreConnection *conn = NULL;
   gchar *port_s;
   gchar *host;
-  gchar *error_msg = NULL;
-  VinagreConnection *conn = NULL;
-
-  VinagreTubeHandlerPrivate *priv = VINAGRE_TUBE_HANDLER_GET_PRIVATE (self);
+  const gchar *service_s;
+  VinagrePlugin *plugin;
 
   if (error != NULL)
     {
-      g_printerr ("Impossible to accept the stream tube: %s\n",
+      error_msg = g_strdup_printf
+          (_("Impossible to get service property: %s"),
           error->message);
+      vinagre_utils_show_error (NULL, (const gchar *) error_msg,
+          GTK_WINDOW (priv->window));
+      g_free (priv->hostname);
+      g_free (error_msg);
       g_signal_emit (G_OBJECT (self), signals[DISCONNECTED], 0);
-      return ;
+      return;
     }
 
-  priv->signal_invalidated_id = g_signal_connect (G_OBJECT (channel),
+  service_s = g_value_get_string (service);
+
+  vinagre_debug_message (DEBUG_TUBE, "service name = %s\n",
+      service_s);
+
+  if (!g_strcmp0 (service_s, "rfb"))
+    service_s = "vnc";
+
+  plugin = vinagre_plugins_engine_get_plugin_by_protocol
+      (vinagre_plugins_engine_get_default (), service_s);
+
+  if (plugin == NULL)
+    {
+      error_msg = g_strdup_printf
+          (_("The protocol %s is not supported."), service_s);
+      vinagre_utils_show_error (NULL, (const gchar *) error_msg,
+          GTK_WINDOW (priv->window));
+      g_free (priv->hostname);
+      g_free (error_msg);
+      g_signal_emit (G_OBJECT (self), signals[DISCONNECTED], 0);
+      return;
+    }
+
+  priv->signal_invalidated_id = g_signal_connect (G_OBJECT (priv->channel),
       "invalidated", G_CALLBACK (vinagre_tube_handler_tube_invalidated),
       self);
 
-  dbus_g_type_struct_get (address, 0, &hostname, 1, &port,
-      G_MAXUINT);
+  port_s = g_strdup_printf ("%u", priv->port);
 
-  port_s = g_strdup_printf ("%u", port);
+  host = g_strconcat (vinagre_plugin_get_protocol (plugin), "://",
+      priv->hostname, ":", port_s,
+      NULL);
 
-  host = g_strconcat (hostname, ":", port_s, NULL);
-
-  conn = vinagre_connection_new_from_string (host, &error_msg, TRUE);
+  vinagre_debug_message (DEBUG_TUBE, "Host = %s\n",
+      host);
 
   g_free (port_s);
-  g_free (hostname);
+  g_free (priv->hostname);
+
+  conn = vinagre_connection_new_from_string (host, &error_conn_msg,
+      TRUE);
+
   g_free (host);
 
   if (conn == NULL)
     {
-      g_printerr ("Impossible to create the connection: %s\n",
-          error_msg);
-      g_signal_emit (G_OBJECT (self), signals[DISCONNECTED], 0);
+      error_msg = g_strdup_printf
+          (_("Impossible to create the connection: %s"), error_conn_msg);
+      vinagre_utils_show_error (NULL, (const gchar *) error_msg,
+          GTK_WINDOW (priv->window));
+      g_signal_handler_disconnect (G_OBJECT (priv->channel),
+        priv->signal_invalidated_id);
+      g_free (error_conn_msg);
       g_free (error_msg);
-      return ;
+      g_signal_emit (G_OBJECT (self), signals[DISCONNECTED], 0);
+      return;
     }
 
   vinagre_cmd_direct_connect (conn, priv->window);
@@ -268,6 +309,37 @@ vinagre_tube_handler_accept_stream_tube_cb (TpChannel *channel,
       (vinagre_tube_handler_tab_disconnected_cb), self);
 }
 
+static void
+vinagre_tube_handler_accept_stream_tube_cb (TpChannel *channel,
+    const GValue *address,
+    const GError *error,
+    gpointer self,
+    GObject *weak_object)
+{
+  gchar *error_msg = NULL;
+  VinagreTubeHandlerPrivate *priv = VINAGRE_TUBE_HANDLER_GET_PRIVATE (self);
+
+  if (error != NULL)
+    {
+      error_msg = g_strdup_printf
+          (_("Impossible to accept the stream tube: %s"),
+          error->message);
+      vinagre_utils_show_error (NULL, (const gchar *) error_msg,
+          GTK_WINDOW (priv->window));
+      g_free (error_msg);
+      g_signal_emit (G_OBJECT (self), signals[DISCONNECTED], 0);
+      return ;
+    }
+
+  dbus_g_type_struct_get (address, 0, &priv->hostname, 1, &priv->port,
+      G_MAXUINT);
+
+  tp_cli_dbus_properties_call_get (channel,
+      -1, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE, "Service",
+      vinagre_tube_handler_call_service_cb,
+      self, NULL, NULL);
+}
+
 static void
 vinagre_tube_handler_dialog_response_cb (GtkDialog *dialog,
     gint response,
@@ -301,50 +373,40 @@ vinagre_tube_handler_contact_get_avatar_filename (TpContact *contact,
     const gchar *token,
     VinagreTubeHandler *self)
 {
-  TpConnection *connection;
   gchar *avatar_path;
   gchar *avatar_file;
   gchar *token_escaped;
-  gchar *contact_escaped;
-  gchar *mc_account_unique_name;
-  GError *error = NULL;
-  DBusGProxy *proxy;
+  TpConnection *connection;
+  gchar *cm;
+  gchar *protocol;
+
+  if (contact == NULL)
+    return NULL;
+
+  token_escaped = tp_escape_as_identifier (token);
 
   connection = tp_contact_get_connection (contact);
 
-  proxy = dbus_g_proxy_new_for_name (tp_get_bus (), MC_DBUS_SERVICE,
-      MC_DBUS_SERVICE_PATH, MC_DBUS_SERVICE);
-
-  /* Have to do that while waiting for mission control 5 because
-  there is no mc API in telepathy-glib */
-  if (!dbus_g_proxy_call (proxy, "GetAccountForConnection", &error,
-      G_TYPE_STRING, tp_proxy_get_object_path (connection),
-      G_TYPE_INVALID,
-      G_TYPE_STRING, &mc_account_unique_name,
-      G_TYPE_INVALID))
+  if (!tp_connection_parse_object_path (connection, &protocol, &cm))
     {
-      g_printerr ("Failed to request name: %s",
-          error ? error->message : "No error given");
-      g_clear_error (&error);
+      vinagre_debug_message (DEBUG_TUBE, "Impossible to parse object path\n");
       return NULL;
     }
 
-  contact_escaped = tp_escape_as_identifier (tp_contact_get_identifier
-      (contact));
-
-  token_escaped = tp_escape_as_identifier (token);
-  connection = tp_contact_get_connection (contact);
-
   avatar_path = g_build_filename (g_get_user_cache_dir (),
-      "Empathy", "avatars", mc_account_unique_name,
-      contact_escaped, NULL);
+      "telepathy",
+      "avatars",
+      cm,
+      protocol,
+      NULL);
+  g_mkdir_with_parents (avatar_path, 0700);
 
   avatar_file = g_build_filename (avatar_path, token_escaped, NULL);
 
-  g_free (contact_escaped);
   g_free (token_escaped);
   g_free (avatar_path);
-  g_object_unref (proxy);
+  g_free (cm);
+  g_free (protocol);
 
   return avatar_file;
 }
@@ -393,10 +455,15 @@ vinagre_tube_handler_factory_handle_cb (TpConnection *connection,
   gchar *filename;
   GtkWidget *image;
   GError *error_failed = NULL;
+  gchar *error_msg = NULL;
 
   if (error != NULL)
     {
-      g_printerr ("Impossible to get the contact name: %s\n", error->message);
+      error_msg = g_strdup_printf
+          (_("Impossible to get the contact name: %s"), error->message);
+      vinagre_utils_show_error (NULL, (const gchar *) error_msg,
+          GTK_WINDOW (priv->window));
+      g_free (error_msg);
       g_signal_emit (G_OBJECT (self), signals[DISCONNECTED], 0);
       return;
     }
@@ -427,8 +494,12 @@ vinagre_tube_handler_factory_handle_cb (TpConnection *connection,
 
           if (pixbuf_view == NULL)
             {
-              g_printerr ("Impossible to get the avatar: %s\n",
+              error_msg = g_strdup_printf
+                  (_("Impossible to get the avatar: %s"),
                   error_failed->message);
+              vinagre_utils_show_error (NULL, (const gchar *) error_msg,
+                  GTK_WINDOW (priv->window));
+              g_free (error_msg);
               image = gtk_image_new_from_icon_name ("stock_person",
                   GTK_ICON_SIZE_DIALOG);
             }
@@ -467,7 +538,7 @@ vinagre_tube_handler_constructed (GObject *object)
   TpContactFeature features[] = { TP_CONTACT_FEATURE_ALIAS,
       TP_CONTACT_FEATURE_AVATAR_TOKEN };
 
-  vinagre_debug_message (DEBUG_TUBE," -- New Tube handler --\n");
+  vinagre_debug_message (DEBUG_TUBE, "-- New Tube handler --\n");
 
   connection = tp_channel_borrow_connection (priv->channel);
 
